import java.io.*;

public class InteractPlayer implements Player {
    private GameRules gr;
    private BufferedReader in;

    public InteractPlayer(GameRules gr) {
	this.gr = gr;
	in = new BufferedReader(new InputStreamReader(System.in));
    }

    public Move getNextMove(GameState gs) {
	System.out.print("Enter next move (h for help, q to quit): ");
	String s;
	try {
	    s = in.readLine();
	}
	catch (IOException e) {
	    System.err.print("Error reading input.\n");
	    return null;
	}
	s = s.toLowerCase();
	if (s.startsWith("q")) return null;
	if (s.startsWith("h")) {
	    System.out.print("Enter a pair of coordinates tx ty to put a new piece on square (tx,ty),\n"+
			     "enter a quadruple fx fy tx ty to move a piece from square (fx,fy) to\n"+
			     "square (tx,ty).\n");
	    return getNextMove(gs);
	}
	Move move;
	try {
	    move = newMove(s);
	}
	catch (IllegalArgumentException e) {
	    System.out.print("Illegal move - try again\n");
	    return getNextMove(gs);
	}
	if (!gr.isLegal(gs, move)) {
	    System.out.print("Illegal move - try again\n");
	    return getNextMove(gs);
	}
	return move;
    }

    /** Constructs a move represented by the string <tt>s</tt>.  This
     * string must contain either two integers <tt>tx ty</tt>, meaning
     * that a game piece is to be placed at this position, or four
     * integers <tt>fx fy tx ty</tt> indicating that the piece should
     * be moved from <tt>fx,fy</tt> to <tt>tx,ty</tt>.  These integers
     * must be the numbers 0, 1 or 2.  All white space is ignored.
     */
    private Move newMove(String s) throws IllegalArgumentException {
	s = s.replaceAll("\\s","");
	if (s.length()==2) {
	    return new Move(Character.getNumericValue(s.charAt(0)),
			    Character.getNumericValue(s.charAt(1)));
	}
	else if (s.length()==4) {
	    return new Move(Character.getNumericValue(s.charAt(0)),
			    Character.getNumericValue(s.charAt(1)),
			    Character.getNumericValue(s.charAt(2)),
			    Character.getNumericValue(s.charAt(3)));
	}
	else throw new IllegalArgumentException();
    }

}
