import java.util.*;

/** An object of this class represents a move that could be applied to
 * a game state.
 */
public class Move {
    /** x-coordinate of square from which piece is being moved, or -1
     * if new piece is just being placed on the board.
     */
    public int fx;
    /** y-coordinate of square from which piece is being moved, or -1
     * if new piece is just being placed on the board.
     */
    public int fy;

    /** x-coordinate of square to which piece is being moved.
     */
    public int tx;

    /** y-coordinate of square to which piece is being moved.
     */
    public int ty;

    /** Constructs a move representing placing a new game piece at position
     * <tt>tx,ty</tt>.
     */
    public Move(int tx, int ty) {
	this(-1,-1,tx,ty);
    }

    /** Constructs a move representing moving a game piece from
     * <tt>fx,fy</tt> to <tt>tx,ty</tt>.
     */
    public Move(int fx, int fy, int tx, int ty) {
	this.fx = fx;
	this.fy = fy;
	this.tx = tx;
	this.ty = ty;
    }

    /** Produces a description of this Move object.
     */
    public String toString() {
	if (fx==-1)
	    return ""+tx+ty;
	else
	    return ""+fx+fy+tx+ty;
    }

    /** Compares the specified object to this move for equality.
     * Returns <tt>true</tt> if the object is also a Move object, and
     * if it represents an identical move.
     * @param o object to be compared
     */
    public boolean equals(Object o) {
	Move m;
	try {
	    m = (Move) o;
	}
	catch (ClassCastException e) {
	    return false;
	}

	return (m.fx == fx && m.fy == fy && m.tx == tx && m.ty == ty);
    }
}
