import java.util.*;
import java.io.*;

public class OptPlayer implements Player {
    private Map moves;
    private GameRules gr;
    private Random random;
    private double prob;
    
    public OptPlayer(String game_id) throws IllegalArgumentException, IOException {
	this(game_id, 1.0);
    }

    public OptPlayer(String game_id, double prob) throws IllegalArgumentException, IOException {
	if (prob<-0.001 || prob>1.001)
	    throw new IllegalArgumentException();
	this.prob = prob;
	random = new Random();
	
	if (!(game_id.equals("centerdiag") ||
	      game_id.equals("alldiag") ||
	      game_id.equals("nodiag")))
	    throw new IllegalArgumentException();
	gr = new GameRules(game_id);
	moves = new HashMap();
	
	BufferedReader in;
	String line;
	boolean waitMove = false;
	GameState gs = new GameState();
	try {
	    in = new BufferedReader(new FileReader(game_id + ".opt"));
	}
	catch (FileNotFoundException e) {
	    System.err.print("File "+game_id+".opt not found.\n");
	    throw e;
	}
	try {
	    while ((line=in.readLine()) != null) {
		line = line.trim();
		if (line.length() == 0) break;
		if (waitMove) {
		    Move m;
		    if (line.length() == 2)
			m = new Move(Character.getNumericValue(line.charAt(0)),
				     Character.getNumericValue(line.charAt(1)));
		    else
			m = new Move(Character.getNumericValue(line.charAt(0)),
				     Character.getNumericValue(line.charAt(1)),
				     Character.getNumericValue(line.charAt(2)),
				     Character.getNumericValue(line.charAt(3)));
		    moves.put(gs, m);
		} else
		    gs = newGameState(line);
		waitMove = !waitMove;
	    }
	}
	catch (IOException e) {
	    System.err.print("Error reading file "+game_id+".opt.\n");
	    throw e;
	}
	catch (IllegalArgumentException e) {
	    System.err.print("File "+game_id+".opt has wrong format.\n");
	    throw e;
	}
	try {
	    in.close();
	}
	catch (IOException e) {
	    System.err.print("Error closing file "+game_id+".opt.\n");
	    throw e;
	}
	if (waitMove) {
	    System.err.print("File "+game_id+".opt has wrong format.\n");
	    throw new IllegalArgumentException();
	}
    }
    
    public Move getNextMove(GameState gs) {
	if (random.nextDouble() < prob)
	    return (Move) moves.get(gs);
	Move[] legalMoves = gr.legalMoves(gs);
	return legalMoves[random.nextInt(legalMoves.length)];
    }

    private static GameState newGameState(String s) {
	LinkedList L = new LinkedList();
	int[][] squares = new int[3][3];
	int turn;

	for (int i=0; i<s.length(); i++) {
	    char ch = s.charAt(i);
	    if (Character.isWhitespace(ch)) continue;
	    L.addLast(new Integer(ch=='+' ? +1 : ch=='-' ? -1 : 0));
	}
	for (int y=2; y>=0; y--)
	    for (int x=0; x<3; x++)
		squares[x][y] = ((Integer) L.removeFirst()).intValue();
	turn = ((Integer) L.removeFirst()).intValue();
	return new GameState(squares, turn);
    }


}
